class ApplicationController < ActionController::Base

  protect_from_forgery
  before_filter :authenticate
  before_filter :set_locale
  helper_method :current_user, :is_allowed_to?

protected

  def render_403
    render(:template => "common/403", :status => 403)
    return(false)
  end

  def require_login
    go_on = true
    if !current_user
      redirect_to(login_url,
                  :notice => t(:notice_login_required),
                  :back_url => url_for(params))
      go_on = false
    end
    return(go_on)
  end

private

  def authenticate
    go_on = true
    if !is_allowed_to?(params[:controller], params[:action])
      render_403
      go_on = false
    end
    return(go_on)
  end

  def set_locale
    if current_user and I18n.available_locales.include?(current_user.locale.to_sym)
      I18n.locale = current_user.locale.to_sym
    elsif (I18n.available_locales.any?{|loc| loc.to_s == extract_locale_from_accept_language_header})
      I18n.locale = extract_locale_from_accept_language_header
    end
  end

  def extract_locale_from_accept_language_header
    request.env['HTTP_ACCEPT_LANGUAGE'].scan(/^[a-z]{2}/).first
  end
  
  def current_user_session
    if (!(defined?(@current_user_session)))
      @current_user_session = UserSession.find
    end
    return(@current_user_session)
  end
  
  def current_user
    @current_user = current_user_session && current_user_session.record
    return(@current_user)
  end

  def require_role(role)
    go_on = true
    if (!require_login)
      go_on = false
    elsif (!(current_user.has_role?(role)))
      render_403
      go_on = false
    end
    return(go_on)
  end
  
  def is_allowed_to?(controller, action)
    is_allowed = false
    begin
      controller_class_name = controller.to_s.camelize + "Controller"
      controller_class = Module.const_get(controller_class_name)
      if controller_class.is_a?(Class)
        controller_object = controller_class.new
        if controller_object.respond_to?("allows_to?")
          is_allowed = controller_object.allows_to?(action.to_s)
        end
      end
    rescue Exception => e
    end
    return(is_allowed)
  end

end
